@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2004 Marco Pesenti Gritti
+ *  Copyright (C) 2003, 2004 Xan Lopez
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: downloader-view.c,v 1.42 2004/01/12 19:59:08 chpe Exp $
+ *  $Id: downloader-view.c,v 1.51 2004/02/01 00:49:46 xan Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -202,8 +203,9 @@ downloader_view_finalize (GObject *object)
 DownloaderView *
 downloader_view_new (void)
 {
-	return EPHY_DOWNLOADER_VIEW (g_object_new
-			(EPHY_TYPE_DOWNLOADER_VIEW, NULL));
+	return EPHY_DOWNLOADER_VIEW (g_object_new (EPHY_TYPE_DOWNLOADER_VIEW,
+			 			   "persist-position", TRUE,
+			 			   NULL));
 }
 
 static char *
@@ -240,7 +242,7 @@ update_download_row (DownloaderView *dv, EphyDownload *download)
 	GtkTreeIter iter;
 	EphyDownloadState state;
 	long total, current, remaining_secs;
-	char *remaining, *file, *cur_progress;
+	char *remaining, *file, *cur_progress, *name;
 	struct tm;
 	int percent;
 
@@ -274,21 +276,20 @@ update_download_row (DownloaderView *dv, EphyDownload *download)
 
 	cur_progress = gnome_vfs_format_file_size_for_display (current);
 
+	name = ephy_download_get_name (download);
+
 	if (total != -1)
 	{
 		char *total_progress;
 
 		total_progress = gnome_vfs_format_file_size_for_display (total);
-		file = g_strdup_printf ("%s\n%s of %s",
-					ephy_download_get_name (download),
+		file = g_strdup_printf ("%s\n%s of %s", name,
 					cur_progress, total_progress);
 		g_free (total_progress);
 	}
 	else
 	{
-		file = g_strdup_printf ("%s\n%s",
-					ephy_download_get_name (download),
-					cur_progress);
+		file = g_strdup_printf ("%s\n%s", name, cur_progress);
 	}
 
 	if (remaining_secs < 0)
@@ -310,6 +311,8 @@ update_download_row (DownloaderView *dv, EphyDownload *download)
 			    -1);
 	gtk_tree_path_free (path);
 
+	g_free (name);
+	g_free (cur_progress);
 	g_free (file);
 	g_free (remaining);
 }
@@ -329,24 +332,45 @@ seconds_remaining_total (EphyDownload *download, gpointer data, DownloaderView *
 static void
 update_status_icon (DownloaderView *dv)
 {
-	char *tooltip, *remaining;
-	int downloads;
+	char *tooltip, *downloadstring, *remainingstring;
+	int downloads, remaining;
 
 	dv->priv->remaining_secs = 0;
 	g_hash_table_foreach (dv->priv->downloads_hash,
 			      (GHFunc) seconds_remaining_total, dv);
 	
-	remaining = format_interval (dv->priv->remaining_secs);
+	remaining = (dv->priv->remaining_secs);
+
+	if (remaining < 60)
+	{
+		remainingstring = g_strdup_printf (ngettext ("About %d second left",
+					"About %d seconds left", remaining),
+					remaining);
+	}
+	else
+	{
+		remaining /= 60;
+		
+		remainingstring = g_strdup_printf (ngettext ("About %d minute left",
+					"About %d minutes left", remaining),
+					remaining);
+	}
+
 	downloads = g_hash_table_size (dv->priv->downloads_hash);
-	tooltip = g_strdup_printf ("%d downloads, with a total of "
-				   "%s minutes remaining",
-				   downloads, remaining);
+
+	downloadstring = g_strdup_printf (ngettext ("%d download", 
+					"%d downloads", downloads), 
+					downloads);
+
+	tooltip = g_strdup_printf ("%s\n%s",
+					downloadstring, remainingstring);
 
 	egg_status_icon_set_tooltip (dv->priv->status_icon,
 				     tooltip, NULL);
 
 	g_free (tooltip);
-	g_free (remaining);
+	g_free (downloadstring);
+	g_free (remainingstring);
 }
 
 static void
@@ -405,8 +429,9 @@ downloader_view_build_ui (DownloaderView *dv)
 
 	ephy_dialog_construct (d,
                                properties,
-                               "epiphany.glade",
-                               "download_manager_dialog");
+			       ephy_file ("epiphany.glade"),
+                               "download_manager_dialog",
+			       NULL);
 
 	/* lookup needed widgets */
 	priv->window = ephy_dialog_get_control(d, properties[PROP_WINDOW].id);
@@ -415,7 +440,7 @@ downloader_view_build_ui (DownloaderView *dv)
 	priv->abort_button = ephy_dialog_get_control (d, properties[PROP_ABORT_BUTTON].id);
 
 	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview)),
-				     GTK_SELECTION_SINGLE);
+				     GTK_SELECTION_BROWSE);
 
 	liststore = gtk_list_store_new (4,
 					G_TYPE_INT,
@@ -562,6 +587,8 @@ downloader_view_remove_download (DownloaderView *dv, EphyDownload *download)
 		gtk_tree_row_reference_free (row_ref);
 	}
 
+	update_status_icon (dv);
+
 	/* Close the dialog if there are no more downloads */
 
 	if (!g_hash_table_size (dv->priv->downloads_hash))
