@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2002, 2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,29 +15,31 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-history.c,v 1.34 2003/10/19 13:29:46 marco Exp $
+ *  $Id: ephy-history.c,v 1.44 2004/01/25 20:01:12 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
-#include "ephy-types.h"
 #include "ephy-history.h"
 #include "ephy-file-helpers.h"
 #include "ephy-debug.h"
+#include "ephy-node-db.h"
 #include "ephy-node-common.h"
+#include "eel-gconf-extensions.h"
+#include "ephy-prefs.h"
 
 #include <time.h>
 #include <string.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 
 #define EPHY_HISTORY_XML_ROOT	 "ephy_history"
 #define EPHY_HISTORY_XML_VERSION "1.0"
 
 /* how often to save the history, in milliseconds */
-#define HISTORY_SAVE_INTERVAL (60 * 5 * 1000)
+#define HISTORY_SAVE_INTERVAL (5 * 60 * 1000)
 
 /* if you change this remember to change also the user interface description */
 #define HISTORY_PAGE_OBSOLETE_DAYS 10
@@ -52,39 +54,41 @@ struct EphyHistoryPrivate
 	EphyNode *pages;
 	EphyNode *last_page;
 	GHashTable *hosts_hash;
-	GStaticRWLock *hosts_hash_lock;
 	GHashTable *pages_hash;
-	GStaticRWLock *pages_hash_lock;
 	int autosave_timeout;
 	guint update_hosts_idle;
+	gboolean dirty;
+	gboolean enabled;
+	guint disable_history_notifier_id;
+};
+
+enum
+{
+	PROP_0,
+	PROP_ENABLED
 };
 
 enum
 {
-        ADD,
-	UPDATE,
-	REMOVE,
 	VISITED,
+	CLEARED,
         LAST_SIGNAL
 };
 
-static void
-ephy_history_class_init (EphyHistoryClass *klass);
-static void
-ephy_history_init (EphyHistory *tab);
-static void
-ephy_history_finalize (GObject *object);
+static guint signals[LAST_SIGNAL] = { 0 };
+
+static void ephy_history_class_init	(EphyHistoryClass *klass);
+static void ephy_history_init		(EphyHistory *history);
+static void ephy_history_finalize	(GObject *object);
 
 static GObjectClass *parent_class = NULL;
 
-static guint ephy_history_signals[LAST_SIGNAL] = { 0 };
-
 GType
 ephy_history_get_type (void)
 {
-        static GType ephy_history_type = 0;
+        static GType type = 0;
 
-        if (ephy_history_type == 0)
+        if (type == 0)
         {
                 static const GTypeInfo our_info =
                 {
@@ -99,12 +103,60 @@ ephy_history_get_type (void)
                         (GInstanceInitFunc) ephy_history_init
                 };
 
-                ephy_history_type = g_type_register_static (G_TYPE_OBJECT,
-							      "EphyHistory",
-							      &our_info, 0);
+                type = g_type_register_static (G_TYPE_OBJECT,
+					       "EphyHistory",
+					       &our_info, 0);
         }
 
-        return ephy_history_type;
+        return type;
+}
+
+static void
+ephy_history_set_enabled (EphyHistory *history,
+			  gboolean enabled)
+{
+	LOG ("ephy_history_set_enabled %d", enabled)
+
+	history->priv->enabled = enabled;
+
+	ephy_node_db_set_immutable (history->priv->db, !enabled);
+
+	if (enabled == FALSE)
+	{
+		ephy_history_clear (history);
+	}
+}
+
+static void
+ephy_history_set_property (GObject *object,
+			   guint prop_id,
+			   const GValue *value,
+			   GParamSpec *pspec)
+{
+	EphyHistory *history = EPHY_HISTORY (history);
+
+	switch (prop_id)
+	{
+		case PROP_ENABLED:
+			ephy_history_set_enabled (history, g_value_get_boolean (value));
+			break;
+	}
+}
+
+static void
+ephy_history_get_property (GObject *object,
+			   guint prop_id,
+			   GValue *value,
+			   GParamSpec *pspec)
+{
+	EphyHistory *history = EPHY_HISTORY (history);
+
+	switch (prop_id)
+	{
+		case PROP_ENABLED:
+			g_value_set_boolean (value, history->priv->enabled);
+			break;
+		}
 }
 
 static void
@@ -115,8 +167,18 @@ ephy_history_class_init (EphyHistoryClass *klass)
         parent_class = g_type_class_peek_parent (klass);
 
         object_class->finalize = ephy_history_finalize;
+	object_class->get_property = ephy_history_get_property;
+	object_class->set_property = ephy_history_set_property;
 
-	ephy_history_signals[VISITED] =
+	g_object_class_install_property (object_class,
+					 PROP_ENABLED,
+					 g_param_spec_boolean ("enabled",
+							       "Enabled",
+							       "Enabled",
+							       TRUE,
+							       G_PARAM_READWRITE));
+
+	signals[VISITED] =
                 g_signal_new ("visited",
                               G_OBJECT_CLASS_TYPE (object_class),
                               G_SIGNAL_RUN_FIRST,
@@ -127,6 +189,16 @@ ephy_history_class_init (EphyHistoryClass *klass)
                               1,
 			      G_TYPE_STRING);
 
+	signals[CLEARED] =
+                g_signal_new ("cleared",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (EphyHistoryClass, cleared),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
+
 	g_type_class_add_private (object_class, sizeof(EphyHistoryPrivate));
 }
 
@@ -159,7 +231,6 @@ remove_obsolete_pages (EphyHistory *eb)
         g_date_set_time (&current_date, time (NULL));
 
 	children = ephy_node_get_children (eb->priv->pages);
-	ephy_node_thaw (eb->priv->pages);
 	for (i = 0; i < children->len; i++)
 	{
 		EphyNode *kid;
@@ -176,46 +247,31 @@ remove_obsolete_pages (EphyHistory *eb)
 static void
 ephy_history_save (EphyHistory *eb)
 {
-	xmlDocPtr doc;
-	xmlNodePtr root;
-	GPtrArray *children;
-	int i;
+	int ret;
 
-	LOG ("Saving history")
-
-	/* save nodes to xml */
-	xmlIndentTreeOutput = TRUE;
-	doc = xmlNewDoc ("1.0");
-
-	root = xmlNewDocNode (doc, NULL, "ephy_history", NULL);
-	xmlSetProp (root, "version", EPHY_HISTORY_XML_VERSION);
-	xmlDocSetRootElement (doc, root);
-
-	children = ephy_node_get_children (eb->priv->hosts);
-	for (i = 0; i < children->len; i++)
+	/* only save if there are changes */
+	if (eb->priv->dirty == FALSE)
 	{
-		EphyNode *kid;
-
-		kid = g_ptr_array_index (children, i);
-		if (kid == eb->priv->pages) continue;
-
-		ephy_node_save_to_xml (kid, root);
+		return;
 	}
-	ephy_node_thaw (eb->priv->hosts);
 
-	children = ephy_node_get_children (eb->priv->pages);
-	for (i = 0; i < children->len; i++)
+	LOG ("Saving history db")
+
+	ret = ephy_node_db_write_to_xml_safe
+		(eb->priv->db, eb->priv->xml_file,
+		 EPHY_HISTORY_XML_ROOT,
+		 EPHY_HISTORY_XML_VERSION,
+		 NULL, /* comment */
+		 eb->priv->hosts,
+		 1, eb->priv->pages,
+		 eb->priv->pages, 0,
+		 NULL);
+
+	if (ret >=0)
 	{
-		EphyNode *kid;
-
-		kid = g_ptr_array_index (children, i);
-
-		ephy_node_save_to_xml (kid, root);
+		/* save was successful */
+		eb->priv->dirty = FALSE;
 	}
-	ephy_node_thaw (eb->priv->pages);
-
-	ephy_file_save_xml (eb->priv->xml_file, doc);
-	xmlFreeDoc(doc);
 }
 
 static void
@@ -223,13 +279,11 @@ hosts_added_cb (EphyNode *node,
 	        EphyNode *child,
 	        EphyHistory *eb)
 {
-	g_static_rw_lock_writer_lock (eb->priv->hosts_hash_lock);
+	eb->priv->dirty = TRUE;
 
 	g_hash_table_insert (eb->priv->hosts_hash,
 			     (char *) ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION),
 			     child);
-
-	g_static_rw_lock_writer_unlock (eb->priv->hosts_hash_lock);
 }
 
 static void
@@ -238,12 +292,18 @@ hosts_removed_cb (EphyNode *node,
 		  guint old_index,
 		  EphyHistory *eb)
 {
-	g_static_rw_lock_writer_lock (eb->priv->hosts_hash_lock);
+	eb->priv->dirty = TRUE;
 
 	g_hash_table_remove (eb->priv->hosts_hash,
 			     ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION));
+}
 
-	g_static_rw_lock_writer_unlock (eb->priv->hosts_hash_lock);
+static void
+hosts_changed_cb (EphyNode *node,
+		  EphyNode *child,
+		  EphyHistory *eb)
+{
+	eb->priv->dirty = TRUE;
 }
 
 static void
@@ -251,13 +311,11 @@ pages_added_cb (EphyNode *node,
 	        EphyNode *child,
 	        EphyHistory *eb)
 {
-	g_static_rw_lock_writer_lock (eb->priv->pages_hash_lock);
+	eb->priv->dirty = TRUE;
 
 	g_hash_table_insert (eb->priv->pages_hash,
 			     (char *) ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION),
 			     child);
-
-	g_static_rw_lock_writer_unlock (eb->priv->pages_hash_lock);
 }
 
 static void
@@ -266,12 +324,18 @@ pages_removed_cb (EphyNode *node,
 		  guint old_index,
 		  EphyHistory *eb)
 {
-	g_static_rw_lock_writer_lock (eb->priv->pages_hash_lock);
+	eb->priv->dirty = TRUE;
 
 	g_hash_table_remove (eb->priv->pages_hash,
 			     ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION));
+}
 
-	g_static_rw_lock_writer_unlock (eb->priv->pages_hash_lock);
+static void
+pages_changed_cb (EphyNode *node,
+		  EphyNode *child,
+		  EphyHistory *eb)
+{
+	eb->priv->dirty = TRUE;
 }
 
 static gboolean
@@ -308,7 +372,6 @@ update_host_on_child_remove (EphyNode *node)
 			new_host_last_visit = last_visit;
 		}
 	}
-	ephy_node_thaw (node);
 
 	if (host_last_visit != new_host_last_visit)
 	{
@@ -348,7 +411,6 @@ update_hosts (EphyHistory *eh)
 			}
 		}
 	}
-	ephy_node_thaw (eh->priv->hosts);
 
 	g_list_foreach (empty, (GFunc)ephy_node_unref, NULL);
 	g_list_free (empty);
@@ -364,7 +426,7 @@ page_removed_from_host_cb (EphyNode *node,
 		           guint old_index,
 		           EphyHistory *eb)
 {
-	if (!eb->priv->update_hosts_idle)
+	if (eb->priv->update_hosts_idle == 0)
 	{
 		eb->priv->update_hosts_idle = g_idle_add
 			((GSourceFunc)update_hosts, eb);
@@ -384,6 +446,16 @@ connect_page_removed_from_host (char *url,
 					 G_OBJECT (eb));
 }
 
+static void
+disable_history_notifier (GConfClient *client,
+			  guint cnxn_id,
+			  GConfEntry *entry,
+			  EphyHistory *history)
+{
+	ephy_history_set_enabled
+		(history, !eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_HISTORY));
+}
+
 static void
 ephy_history_init (EphyHistory *eb)
 {
@@ -392,6 +464,7 @@ ephy_history_init (EphyHistory *eb)
 
         eb->priv = EPHY_HISTORY_GET_PRIVATE (eb);
 	eb->priv->update_hosts_idle = 0;
+	eb->priv->enabled = TRUE;
 
 	db = ephy_node_db_new (EPHY_NODE_DB_HISTORY);
 	eb->priv->db = db;
@@ -402,13 +475,8 @@ ephy_history_init (EphyHistory *eb)
 
 	eb->priv->pages_hash = g_hash_table_new (g_str_hash,
 			                          g_str_equal);
-	eb->priv->pages_hash_lock = g_new0 (GStaticRWLock, 1);
-	g_static_rw_lock_init (eb->priv->pages_hash_lock);
-
 	eb->priv->hosts_hash = g_hash_table_new (g_str_hash,
 			                         g_str_equal);
-	eb->priv->hosts_hash_lock = g_new0 (GStaticRWLock, 1);
-	g_static_rw_lock_init (eb->priv->hosts_hash_lock);
 
 	/* Pages */
 	eb->priv->pages = ephy_node_new_with_id (db, PAGES_NODE_ID);
@@ -436,6 +504,10 @@ ephy_history_init (EphyHistory *eb)
 					 EPHY_NODE_CHILD_REMOVED,
 				         (EphyNodeCallback) pages_removed_cb,
 					 G_OBJECT (eb));
+	ephy_node_signal_connect_object (eb->priv->pages,
+					 EPHY_NODE_CHILD_CHANGED,
+				         (EphyNodeCallback) pages_changed_cb,
+					 G_OBJECT (eb));
 
 	/* Hosts */
 	eb->priv->hosts = ephy_node_new_with_id (db, HOSTS_NODE_ID);
@@ -448,6 +520,10 @@ ephy_history_init (EphyHistory *eb)
 					 EPHY_NODE_CHILD_REMOVED,
 				         (EphyNodeCallback) hosts_removed_cb,
 					 G_OBJECT (eb));
+	ephy_node_signal_connect_object (eb->priv->hosts,
+					 EPHY_NODE_CHILD_CHANGED,
+				         (EphyNodeCallback) hosts_changed_cb,
+					 G_OBJECT (eb));
 
 	ephy_node_add_child (eb->priv->hosts, eb->priv->pages);
 
@@ -459,11 +535,19 @@ ephy_history_init (EphyHistory *eb)
 			      (GHFunc) connect_page_removed_from_host,
 			      eb);
 
+	/* mark as clean */
+	eb->priv->dirty = FALSE;
+
 	/* setup the periodic history saving callback */
 	eb->priv->autosave_timeout =
 		g_timeout_add (HISTORY_SAVE_INTERVAL,
 		       (GSourceFunc)periodic_save_cb,
 		       eb);
+
+	disable_history_notifier (NULL, 0, NULL, eb);
+	eb->priv->disable_history_notifier_id = eel_gconf_notification_add
+		(CONF_LOCKDOWN_DISABLE_HISTORY,
+		 (GConfClientNotifyFunc) disable_history_notifier, eb);
 }
 
 static void
@@ -484,12 +568,14 @@ ephy_history_finalize (GObject *object)
 	g_object_unref (eb->priv->db);
 
 	g_hash_table_destroy (eb->priv->pages_hash);
-	g_static_rw_lock_free (eb->priv->pages_hash_lock);
 	g_hash_table_destroy (eb->priv->hosts_hash);
-	g_static_rw_lock_free (eb->priv->hosts_hash_lock);
 
 	g_source_remove (eb->priv->autosave_timeout);
 
+	eel_gconf_notification_remove (eb->priv->disable_history_notifier_id);
+
+	g_free (eb->priv->xml_file);
+
 	LOG ("Global history finalized");
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -498,11 +584,7 @@ ephy_history_finalize (GObject *object)
 EphyHistory *
 ephy_history_new ()
 {
-	EphyHistory *eh;
-
-	eh = EPHY_HISTORY (g_object_new (EPHY_TYPE_HISTORY, NULL));
-
-	return eh;
+	return EPHY_HISTORY (g_object_new (EPHY_TYPE_HISTORY, NULL));
 }
 
 static void
@@ -535,20 +617,34 @@ ephy_history_host_visited (EphyHistory *eh,
 
 static EphyNode *
 ephy_history_add_host (EphyHistory *eh, EphyNode *page)
+{
+	const char *url;
+
+	url = ephy_node_get_property_string
+		(page, EPHY_NODE_PAGE_PROP_LOCATION);
+
+	return ephy_history_get_host (eh, url);
+}
+
+EphyNode *
+ephy_history_get_host (EphyHistory *eh, const char *url)
 {
 	GnomeVFSURI *vfs_uri = NULL;
 	EphyNode *host = NULL;
 	const char *host_name = NULL;
 	GList *host_locations = NULL, *l;
 	GValue value = { 0, };
-	const char *url;
 	const char *scheme = NULL;
 	GTime now;
 
-	now = time (NULL);
+	g_return_val_if_fail (url != NULL, NULL);
 
-	url = ephy_node_get_property_string
-		(page, EPHY_NODE_PAGE_PROP_LOCATION);
+	if (eh->priv->enabled == FALSE)
+	{
+		return NULL;
+	}
+
+	now = time (NULL);
 
 	vfs_uri = gnome_vfs_uri_new (url);
 
@@ -596,15 +692,12 @@ ephy_history_add_host (EphyHistory *eh, EphyNode *page)
 
 	g_return_val_if_fail (host_locations != NULL, NULL);
 
-	g_static_rw_lock_reader_lock (eh->priv->hosts_hash_lock);
-
 	for (l = host_locations; l != NULL; l = l->next)
 	{
 		host = g_hash_table_lookup (eh->priv->hosts_hash,
 					    (char *)l->data);
 		if (host) break;
 	}
-	g_static_rw_lock_reader_unlock (eh->priv->hosts_hash_lock);
 
 	if (!host)
 	{
@@ -697,7 +790,7 @@ ephy_history_visited (EphyHistory *eh, EphyNode *node)
 
 	eh->priv->last_page = node;
 
-	g_signal_emit (G_OBJECT (eh), ephy_history_signals[VISITED], 0, url);
+	g_signal_emit (G_OBJECT (eh), signals[VISITED], 0, url);
 }
 
 int
@@ -725,6 +818,11 @@ ephy_history_add_page (EphyHistory *eb,
 	EphyNode *bm, *node, *host;
 	GValue value = { 0, };
 
+	if (eb->priv->enabled == FALSE)
+	{
+		return;
+	}
+
 	node = ephy_history_get_page (eb, url);
 	if (node)
 	{
@@ -762,9 +860,7 @@ ephy_history_get_page (EphyHistory *eb,
 {
 	EphyNode *node;
 
-	g_static_rw_lock_reader_lock (eb->priv->pages_hash_lock);
 	node = g_hash_table_lookup (eb->priv->pages_hash, url);
-	g_static_rw_lock_reader_unlock (eb->priv->pages_hash_lock);
 
 	return node;
 }
@@ -789,7 +885,7 @@ ephy_history_set_page_title (EphyHistory *gh,
 	if (title == NULL || title[0] == '\0') return;
 
 	node = ephy_history_get_page (gh, url);
-	if (!node) return;
+	if (node == NULL) return;
 
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, title);
@@ -845,12 +941,19 @@ ephy_history_clear (EphyHistory *gh)
 {
 	EphyNode *node;
 
+	LOG ("clearing history")
+
+	ephy_node_db_set_immutable (gh->priv->db, FALSE);
+
 	while ((node = ephy_node_get_nth_child (gh->priv->pages, 0)) != NULL)
 	{
 		ephy_node_unref (node);
 	}
-
 	ephy_history_save (gh);
+
+	ephy_node_db_set_immutable (gh->priv->db, !gh->priv->enabled);
+
+	g_signal_emit (gh, signals[CLEARED], 0);
 }
 
 EphyNode *
@@ -873,3 +976,11 @@ ephy_history_get_last_page (EphyHistory *gh)
 	return ephy_node_get_property_string
 		(gh->priv->last_page, EPHY_NODE_PAGE_PROP_LOCATION);
 }
+
+gboolean
+ephy_history_is_enabled (EphyHistory *history)
+{
+	g_return_val_if_fail (EPHY_IS_HISTORY (history), FALSE);
+
+	return history->priv->enabled;
+}
