@@ -34,12 +34,29 @@
  * the provisions above, a recipient may use your version of this file under
  * the terms of any one of the MPL, the GPL or the LGPL.
  *
- * ***** END LICENSE BLOCK ***** */
+ * ***** END LICENSE BLOCK *****
+ *
+ * $Id: EphyHeaderSniffer.cpp,v 1.18 2004/01/29 18:00:33 chpe Exp $
+ */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include "MozillaPrivate.h"
 
 #include "MozDownload.h"
 #include "EphyHeaderSniffer.h"
 #include "netCore.h"
 
+#include "ephy-embed-single.h"
+#include "ephy-embed-shell.h"
+#include "ephy-file-chooser.h"
+#include "ephy-prefs.h"
+#include "ephy-gui.h"
+#include "eel-gconf-extensions.h"
+#include "ephy-debug.h"
+
 #include "nsReadableUtils.h"
 #include "nsIChannel.h"
 #include "nsIHttpChannel.h"
@@ -51,29 +68,48 @@
 #include "nsIDOMHTMLDocument.h"
 #include "nsIDownload.h"
 
-const char* const persistContractID = "@mozilla.org/embedding/browser/nsWebBrowserPersist;1";
+#include <glib/gi18n.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 
 EphyHeaderSniffer::EphyHeaderSniffer (nsIWebBrowserPersist* aPersist, MozillaEmbedPersist *aEmbedPersist,
-		nsIFile* aFile, nsIURI* aURL, nsIDOMDocument* aDocument, nsIInputStream* aPostData,
-                const nsAString& aSuggestedFilename, PRBool aBypassCache)
+		nsIFile* aFile, nsIURI* aURL, nsIDOMDocument* aDocument, nsIInputStream* aPostData)
 : mPersist(aPersist)
 , mEmbedPersist(aEmbedPersist)
 , mTmpFile(aFile)
 , mURL(aURL)
+, mOriginalURI(nsnull)
 , mDocument(aDocument)
 , mPostData(aPostData)
-, mDefaultFilename(aSuggestedFilename)
-, mBypassCache(aBypassCache)
 {
 	mPrompt = do_GetService("@mozilla.org/embedcomp/prompt-service;1");
+
+	LOG ("EphyHeaderSniffer constructor")
 }
 
 EphyHeaderSniffer::~EphyHeaderSniffer()
 {
+	LOG ("EphyHeaderSniffer destructor")
 }
 
 NS_IMPL_ISUPPORTS2(EphyHeaderSniffer, nsIWebProgressListener, nsIAuthPrompt)
 
+NS_IMETHODIMP
+EphyHeaderSniffer::HandleContent ()
+{
+	EphyEmbedSingle *single;
+	gboolean handled = FALSE;
+	nsCString uriSpec;
+
+	if (mPostData) return NS_ERROR_FAILURE;
+
+	mURL->GetSpec (uriSpec);	
+	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (embed_shell));
+	g_signal_emit_by_name (single, "handle_content", mContentType.get(),
+			       uriSpec.get(), &handled);
+
+	return handled ? NS_OK : NS_ERROR_FAILURE;
+}
+
 NS_IMETHODIMP 
 EphyHeaderSniffer::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRUint32 aStateFlags, 
                                   PRUint32 aStatus)
@@ -109,6 +145,9 @@ EphyHeaderSniffer::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aReq
 			mTmpFile->Remove(PR_FALSE);
 		}
 
+		rv = HandleContent ();
+		if (NS_SUCCEEDED (rv)) return NS_OK;
+
 		rv = PerformSave(origURI);
 		if (NS_FAILED(rv))
 		{
@@ -154,119 +193,154 @@ EphyHeaderSniffer::OnSecurityChange (nsIWebProgress *aWebProgress, nsIRequest *a
 	return NS_OK;
 }
 
+static void
+filechooser_response_cb (EphyFileChooser *dialog, gint response, EphyHeaderSniffer* sniffer)
+{
+	if (response == EPHY_RESPONSE_SAVE)
+	{
+		char *filename;
+		GtkWidget *parent = NULL; // FIXME!
+
+		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+
+		LOG ("Filename %s", filename)
+
+		if (filename &&
+		    ephy_gui_confirm_overwrite_file (parent, filename) == TRUE)
+		{
+			nsCOMPtr<nsILocalFile> destFile = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
+			if (destFile)
+			{
+				destFile->InitWithNativePath (nsDependentCString (filename));
+			
+				sniffer->InitiateDownload (destFile);
+			}
+		}
+	}
+
+	// FIXME how to inform user of failed save ?
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
 nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 {
 	nsresult rv;
+	char *path, *download_dir;
+	EmbedPersistFlags flags;
+	PRBool askDownloadDest;
 
-	PRBool isHTML = (mDocument && mContentType.Equals("text/html") ||
-			 mContentType.Equals("text/xml") ||
-			 mContentType.Equals("application/xhtml+xml"));
- 
-        nsCOMPtr<nsILocalFile> file;
-       	rv = NS_NewLocalFile(mDefaultFilename, PR_TRUE, getter_AddRefs(file)); 
-        if (NS_FAILED(rv) || !file) return G_FAILED;
-  
-	nsCOMPtr<nsISupports> sourceData;
-	if (isHTML)
+	mOriginalURI = inOriginalURI;
+
+	flags = ephy_embed_persist_get_flags (EPHY_EMBED_PERSIST (mEmbedPersist));
+	askDownloadDest = flags & EMBED_PERSIST_ASK_DESTINATION;
+
+	nsAutoString defaultFileName;
+
+	if (defaultFileName.IsEmpty() && !mContentDisposition.IsEmpty())
 	{
-		sourceData = do_QueryInterface(mDocument);
+		/* 1 Use the HTTP header suggestion. */
+		PRInt32 index = mContentDisposition.Find("filename=");
+		if (index >= 0)
+		{
+			/* Take the substring following the prefix. */
+			index += strlen ("filename=");
+			nsCAutoString filename;
+			mContentDisposition.Right(filename, mContentDisposition.Length() - index);
+			defaultFileName = NS_ConvertUTF8toUCS2(filename);
+		}
 	}
-	else
+    
+	if (defaultFileName.IsEmpty())
 	{
-		sourceData = do_QueryInterface(mURL);
+		/* 2 For file URLs, use the file name. */
+
+		nsCOMPtr<nsIURL> url(do_QueryInterface(mURL));
+		if (url)
+		{
+			nsCAutoString fileNameCString;
+			url->GetFileName(fileNameCString);
+			defaultFileName = NS_ConvertUTF8toUCS2(fileNameCString);
+		}
+	}
+    
+	if (defaultFileName.IsEmpty() && mDocument)
+	{
+		/* 3 Use the title of the document. */
+
+		nsCOMPtr<nsIDOMHTMLDocument> htmlDoc(do_QueryInterface(mDocument));
+		if (htmlDoc)
+		{
+			htmlDoc->GetTitle(defaultFileName);
+		}
+	}
+    
+	if (defaultFileName.IsEmpty() && mURL)
+	{
+		/* 4 Use the host. */
+		nsCAutoString hostName;
+		mURL->GetHost(hostName);
+		defaultFileName = NS_ConvertUTF8toUCS2(hostName);
+	}
+    
+	/* 5 One last case to handle about:blank and other untitled pages. */
+	if (defaultFileName.IsEmpty())
+	{
+		defaultFileName = NS_ConvertUTF8toUCS2 (_("Untitled"));
+	}
+        
+	/* Validate the file name to ensure legality. */
+	for (PRUint32 i = 0; i < defaultFileName.Length(); i++)
+	{
+		if (defaultFileName[i] == ':' || defaultFileName[i] == '/')
+		{
+			defaultFileName.SetCharAt(i, PRUnichar(' '));
+		}
 	}
 
-	return InitiateDownload(sourceData, file, inOriginalURI);
+	const char *key;
+	key = ephy_embed_persist_get_persist_key (EPHY_EMBED_PERSIST (mEmbedPersist));
+
+	if (askDownloadDest)
+	{
+		EphyFileChooser *dialog;
+		GtkWindow *window;
+		const char *title;
+		int response;
+
+		title = ephy_embed_persist_get_fc_title (EPHY_EMBED_PERSIST (mEmbedPersist));
+		window = ephy_embed_persist_get_fc_parent (EPHY_EMBED_PERSIST (mEmbedPersist));
+
+		dialog = ephy_file_chooser_new (title ? title: _("Save"),
+						GTK_WIDGET (window),
+						GTK_FILE_CHOOSER_ACTION_SAVE,
+						key ? key : CONF_STATE_DOWNLOAD_DIR);
+
+		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog),
+						   NS_ConvertUCS2toUTF8 (defaultFileName).get());
+
+		g_signal_connect (dialog, "response",
+				  G_CALLBACK (filechooser_response_cb), this);
+
+		gtk_widget_show (GTK_WIDGET (dialog));
+
+		return NS_OK;
+	}
+	/* FIXME: how to inform user of failed save ? */
+
+	nsCOMPtr<nsILocalFile> destFile;
+	BuildDownloadPath (NS_ConvertUCS2toUTF8 (defaultFileName).get(), getter_AddRefs (destFile));
+	NS_ENSURE_TRUE (destFile, NS_ERROR_FAILURE);
+
+	return InitiateDownload (destFile);
 }
 
-nsresult EphyHeaderSniffer::InitiateDownload (nsISupports* inSourceData, nsILocalFile* inDestFile,
-					      nsIURI* inOriginalURI)
+nsresult EphyHeaderSniffer::InitiateDownload (nsILocalFile *aDestFile)
 {
-	nsresult rv = NS_OK;
-
-	nsCOMPtr<nsIWebBrowserPersist> webPersist = do_CreateInstance(persistContractID, &rv);
-	if (NS_FAILED(rv)) return rv;
-  
-	nsCOMPtr<nsIURI> sourceURI = do_QueryInterface(inSourceData);
-
-	PRInt64 timeNow = PR_Now();
-  
-	nsAutoString fileDisplayName;
-	inDestFile->GetLeafName(fileDisplayName);
-
-	MozDownload *downloader = new MozDownload ();
-	/* dlListener attaches to its progress dialog here, which gains ownership */
-	rv = downloader->InitForEmbed (inOriginalURI, inDestFile, fileDisplayName.get(),
-				       nsnull, timeNow, webPersist, mEmbedPersist);
-	if (NS_FAILED(rv)) return rv;
-
-	PRInt32 flags = nsIWebBrowserPersist::PERSIST_FLAGS_NO_CONVERSION | 
-                        nsIWebBrowserPersist::PERSIST_FLAGS_REPLACE_EXISTING_FILES;
-	if (mBypassCache)
-	{
-		flags |= nsIWebBrowserPersist::PERSIST_FLAGS_BYPASS_CACHE;
-	}
-	else
-	{
-		flags |= nsIWebBrowserPersist::PERSIST_FLAGS_FROM_CACHE;
-	}
-
-	webPersist->SetPersistFlags(flags);
-    
-	if (sourceURI)
-	{
-		rv = webPersist->SaveURI (sourceURI, nsnull, nsnull,
-					  mPostData, nsnull, inDestFile);
-	}
-	else
-	{
-		nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(inSourceData, &rv);
-		if (!domDoc) return rv;  /* should never happen */
-    
-		PRInt32 encodingFlags = 0;
-		nsCOMPtr<nsILocalFile> filesFolder;
-    
-		if (!mContentType.Equals("text/plain"))
-		{
-			/* Create a local directory in the same dir as our file.  It
-			   will hold our associated files. */
-
-			filesFolder = do_CreateInstance("@mozilla.org/file/local;1");
-			nsAutoString unicodePath;
-			inDestFile->GetPath(unicodePath);
-			filesFolder->InitWithPath(unicodePath);
-      
-			nsAutoString leafName;
-			filesFolder->GetLeafName(leafName);
-			nsAutoString nameMinusExt(leafName);
-			PRInt32 index = nameMinusExt.RFind(".");
-			if (index >= 0)
-			{
-				nameMinusExt.Left(nameMinusExt, index);
-			}
-
-			nameMinusExt += NS_LITERAL_STRING(" Files");
-			filesFolder->SetLeafName(nameMinusExt);
-			PRBool exists = PR_FALSE;
-			filesFolder->Exists(&exists);
-			if (!exists)
-			{
-				rv = filesFolder->Create(nsILocalFile::DIRECTORY_TYPE, 0755);
-				if (NS_FAILED(rv)) return rv;
-			}
-		}
-		else
-		{
-			encodingFlags |= nsIWebBrowserPersist::ENCODE_FLAGS_FORMATTED |
-					 nsIWebBrowserPersist::ENCODE_FLAGS_ABSOLUTE_LINKS |
-					 nsIWebBrowserPersist::ENCODE_FLAGS_NOFRAMES_CONTENT;
-		}
-
-		rv = webPersist->SaveDocument (domDoc, inDestFile, filesFolder,
-					       mContentType.get(), encodingFlags, 80);
-	}
-  
-	return rv;
+	LOG ("Initiating download")
+	return InitiateMozillaDownload (mDocument, mURL, aDestFile,
+					mContentType.get(), mOriginalURI, mEmbedPersist,
+					mPostData, nsnull);
 }
 
 NS_IMETHODIMP EphyHeaderSniffer::Prompt (const PRUnichar *dialogTitle, const PRUnichar *text,
