@@ -71,7 +71,9 @@
 #include <libgnome/gnome-i18n.h>
 
 #include "GtkNSSDialogs.h"
+#include "ephy-file-helpers.h"
 #include "ephy-glade.h"
+#include "ephy-gui.h"
 
 #include <time.h>
 
@@ -1252,9 +1254,9 @@ GtkNSSDialogs::ViewCert(nsIInterfaceRequestor *ctx,
 	PRUnichar ** usage;
 	GtkSizeGroup * sizegroup;
 
-	gxml = ephy_glade_widget_new ("certificate-dialogs.glade", 
+	gxml = ephy_glade_widget_new (ephy_file ("certificate-dialogs.glade"),
 				      "viewcert_dialog",
-				      &dialog, NULL);
+				      &dialog, NULL, NULL);
 
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWidget *gparent = MozillaFindGtkParent (parent);
@@ -1390,7 +1392,19 @@ GtkNSSDialogs::ViewCert(nsIInterfaceRequestor *ctx,
 	g_object_unref (gxml);
 
 	gtk_widget_show_all (dialog);
-	gtk_dialog_run (GTK_DIALOG (dialog));
+
+	int res;
+	while (1)
+	{
+		res = gtk_dialog_run (GTK_DIALOG (dialog));
+		if (res == GTK_RESPONSE_HELP)
+		{
+			ephy_gui_help (GTK_WINDOW (dialog), "epiphany", "using-certificate-viewer");
+			continue;
+		}  
+		break;
+	}
+
 	gtk_widget_destroy (dialog);
 	return NS_OK;
 }
