@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-persist.cpp,v 1.25 2003/11/22 11:32:32 marco Exp $
+ *  $Id: mozilla-embed-persist.cpp,v 1.27 2004/01/14 11:39:31 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +36,7 @@
 #include <nsNetUtil.h>
 #include <nsIHistoryEntry.h>
 #include <nsISHEntry.h>
+#include <nsIDocumentEncoder.h>
 
 static void
 mozilla_embed_persist_class_init (MozillaEmbedPersistClass *klass);
@@ -116,7 +117,7 @@ static void
 impl_cancel (EphyEmbedPersist *persist)
 {
 	nsCOMPtr<nsIWebBrowserPersist> bpersist =
-	MOZILLA_EMBED_PERSIST (persist)->priv->mPersist;
+		MOZILLA_EMBED_PERSIST (persist)->priv->mPersist;
 	if (bpersist)
 	{
 		bpersist->CancelSave ();
@@ -154,18 +155,21 @@ impl_save (EphyEmbedPersist *persist)
 	EphyBrowser *browser = NULL;
 	if (embed)
 	{
-	        browser = (EphyBrowser *) _mozilla_embed_get_ephy_browser (MOZILLA_EMBED(embed));
-		g_return_val_if_fail (browser != NULL, FALSE);
+		browser = (EphyBrowser *) _mozilla_embed_get_ephy_browser (MOZILLA_EMBED(embed));
+		NS_ENSURE_TRUE (browser, FALSE);
 	}
-
 	/* we must have one of uri or browser */
 	g_assert (browser != NULL || uri != NULL);
 
 	/* Get a temp filename to save to */
-	nsCOMPtr<nsIProperties> dirService(do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv));
-	if (!dirService) return FALSE;
+	nsCOMPtr<nsIProperties> dirService(do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID));
+	NS_ENSURE_TRUE (dirService, FALSE);
+
 	nsCOMPtr<nsIFile> tmpFile;
 	dirService->Get("TmpD", NS_GET_IID(nsIFile), getter_AddRefs(tmpFile));
+	if (!tmpFile) return FALSE;
+
+	/* FIXME: make this filename non-guessable */
 	static short unsigned int tmpRandom = 0;
 	nsAutoString tmpNo;
 	tmpNo.AppendInt(tmpRandom++);
@@ -184,10 +188,10 @@ impl_save (EphyEmbedPersist *persist)
 	else
 	{
 		rv = browser->GetDocumentUrl (sURI);
-		if (NS_FAILED(rv)) return FALSE;
+		NS_ENSURE_SUCCESS (rv, FALSE);
 	}
-      	rv = NS_NewURI(getter_AddRefs(inURI), sURI);
-	if (NS_FAILED(rv) || !inURI) return FALSE;
+      	NS_NewURI(getter_AddRefs(inURI), sURI);
+	if (!inURI) return FALSE;
 
 	/* Get post data */
 	nsCOMPtr<nsIInputStream> postData;
@@ -214,13 +218,13 @@ impl_save (EphyEmbedPersist *persist)
 	{		
 		if (flags & EMBED_PERSIST_MAINDOC)
 		{
-                	rv = browser->GetDocument (getter_AddRefs(DOMDocument));
+                	browser->GetDocument (getter_AddRefs(DOMDocument));
 		}
         	else
 		{
-                	rv = browser->GetTargetDocument (getter_AddRefs(DOMDocument));
+                	browser->GetTargetDocument (getter_AddRefs(DOMDocument));
 		}
-        	if (NS_FAILED(rv) || !DOMDocument) return FALSE;
+        	NS_ENSURE_TRUE (DOMDocument, FALSE);
 	}
 
 
@@ -228,7 +232,8 @@ impl_save (EphyEmbedPersist *persist)
 	nsCOMPtr<nsISupports> pageDescriptor;
 	if (flags & EMBED_PERSIST_COPY_PAGE)
 	{
-	        rv = browser->GetPageDescriptor(getter_AddRefs(pageDescriptor));
+	        browser->GetPageDescriptor(getter_AddRefs(pageDescriptor));
+		NS_ENSURE_TRUE (pageDescriptor, FALSE);
 	}
 
 	if (filename == NULL)
@@ -236,7 +241,7 @@ impl_save (EphyEmbedPersist *persist)
 		/* Create an header sniffer and do the save */
 		nsCOMPtr<nsIWebBrowserPersist> webPersist =
 			MOZILLA_EMBED_PERSIST (persist)->priv->mPersist;
-		if (!webPersist) return FALSE;
+		NS_ENSURE_TRUE (webPersist, FALSE);
 
 		EphyHeaderSniffer* sniffer = new EphyHeaderSniffer
 			(webPersist, MOZILLA_EMBED_PERSIST (persist),
@@ -251,9 +256,9 @@ impl_save (EphyEmbedPersist *persist)
 	{
 		/* Filename to save to */
 		nsCOMPtr<nsILocalFile> destFile;
-		rv = NS_NewNativeLocalFile (nsDependentCString(filename),
-				            PR_TRUE, getter_AddRefs(destFile));
-	        if (NS_FAILED(rv) || !destFile) return FALSE;
+		NS_NewNativeLocalFile (nsDependentCString(filename),
+				       PR_TRUE, getter_AddRefs(destFile));
+	        NS_ENSURE_TRUE (destFile, FALSE);
 
 		rv =  InitiateMozillaDownload (DOMDocument, inURI, destFile,
 					       nsnull, inURI, MOZILLA_EMBED_PERSIST (persist),
@@ -264,6 +269,53 @@ impl_save (EphyEmbedPersist *persist)
 	return TRUE;
 }
 
+static char *
+impl_to_string (EphyEmbedPersist *persist)
+{
+	EphyEmbed *embed;
+	nsCOMPtr<nsIDOMDocument> DOMDocument;
+	EmbedPersistFlags flags;
+	EphyBrowser *browser;
+	nsresult rv = NS_OK;
+
+	g_object_ref (persist);
+	
+	g_object_get (persist,
+	              "flags", &flags,
+	              "embed", &embed,
+	              NULL);
+	g_object_unref (persist);
+	g_assert (embed != NULL);
+
+	browser = (EphyBrowser *) _mozilla_embed_get_ephy_browser (MOZILLA_EMBED(embed));
+	g_assert (browser != NULL);
+
+	if (flags & EMBED_PERSIST_MAINDOC)
+	{
+		rv = browser->GetDocument (getter_AddRefs(DOMDocument));
+	}
+       	else
+	{
+		rv = browser->GetTargetDocument (getter_AddRefs(DOMDocument));
+	}
+	if (NS_FAILED(rv) || !DOMDocument) return NULL;
+
+	nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
+	if(!doc) return NULL;
+
+	nsCOMPtr<nsIDocumentEncoder> encoder = do_CreateInstance(NS_DOC_ENCODER_CONTRACTID_BASE "text/html");
+	NS_ENSURE_SUCCESS(rv, NULL);
+
+	rv = encoder->Init(doc, NS_LITERAL_STRING("text/html"), nsIDocumentEncoder::OutputAbsoluteLinks);
+	NS_ENSURE_SUCCESS(rv, NULL);
+
+	nsAutoString aOutputString;
+	encoder->EncodeToString(aOutputString);
+
+	return g_strdup (NS_ConvertUCS2toUTF8(aOutputString).get());
+}
+
+
 static GObject *
 mozilla_embed_persist_constructor (GType type, guint n_construct_properties,
 			           GObjectConstructParam *construct_params)
@@ -288,6 +340,7 @@ mozilla_embed_persist_class_init (MozillaEmbedPersistClass *klass)
 
 	persist_class->save = impl_save;
 	persist_class->cancel = impl_cancel;
+	persist_class->to_string = impl_to_string;
 
 	g_type_class_add_private (object_class, sizeof(MozillaEmbedPersistPrivate));
 }
