@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node-db.c,v 1.11 2004/01/10 16:35:46 chpe Exp $
+ *  $Id: ephy-node-db.c,v 1.12 2004/01/25 16:38:49 chpe Exp $
  */
 
 #include "ephy-node-db.h"
@@ -196,7 +196,7 @@ ephy_node_db_finalize (GObject *object)
 		g_hash_table_destroy (ephy_node_databases);
 	}
 
-	g_ptr_array_free (db->priv->id_to_node, FALSE);
+	g_ptr_array_free (db->priv->id_to_node, TRUE);
 
 	g_free (db->priv->name);
 
