@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tree-model-node.c,v 1.11 2003/10/06 22:20:19 marco Exp $
+ *  $Id: ephy-tree-model-node.c,v 1.14 2004/01/25 20:05:39 chpe Exp $
  */
 
 #include <config.h>
@@ -216,8 +216,6 @@ filter_changed_cb (EphyNodeFilter *filter,
 						g_ptr_array_index (kids, i),
 						i);
 	}
-
-	ephy_node_thaw (model->priv->root);
 }
 
 static void
@@ -338,6 +336,7 @@ ephy_tree_model_node_finalize (GObject *object)
 {
 	EphyTreeModelNode *model = EPHY_TREE_MODEL_NODE (object);
 
+	g_ptr_array_foreach (model->priv->columns, (GFunc) g_free, NULL);
 	g_ptr_array_free (model->priv->columns, TRUE);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -490,7 +489,7 @@ ephy_tree_model_node_get_value (GtkTreeModel *tree_model,
 static guint
 ephy_tree_model_node_get_flags (GtkTreeModel *tree_model)
 {
-	return 0;
+	return GTK_TREE_MODEL_ITERS_PERSIST | GTK_TREE_MODEL_LIST_ONLY;
 }
 
 static gboolean
