@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.c,v 1.162 2004/01/12 19:31:40 chpe Exp $
+ *  $Id: ephy-window.c,v 1.167 2004/01/28 16:20:56 Bordoley Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -78,6 +78,7 @@ static GtkActionEntry ephy_menu_entries [] = {
 	{ "View", NULL, N_("_View") },
 	{ "Bookmarks", NULL, N_("_Bookmarks") },
 	{ "Go", NULL, N_("_Go") },
+	{ "Tools", NULL, N_("T_ools") },
 	{ "Tabs", NULL, N_("_Tabs") },
 	{ "Help", NULL, N_("_Help") },
 	{ "PopupAction", NULL, "" },
@@ -271,9 +272,9 @@ static GtkActionEntry ephy_popups_entries [] = {
 	  NULL, G_CALLBACK (popup_cmd_link_in_new_window) },
 	{ "OpenLinkInNewTab", NULL, N_("Open Link in New _Tab"),
 	  NULL, NULL, G_CALLBACK (popup_cmd_link_in_new_tab) },
-	{ "DownloadLink", GTK_STOCK_SAVE, N_("_Download Link"), NULL,
+	{ "DownloadLink", EPHY_STOCK_DOWNLOAD, N_("_Download Link"), NULL,
 	  NULL, G_CALLBACK (popup_cmd_download_link) },
-	{ "DownloadLinkAs", GTK_STOCK_SAVE, N_("_Download Link As..."), NULL,
+	{ "DownloadLinkAs", GTK_STOCK_SAVE_AS, N_("_Save Link As..."), NULL,
 	  NULL, G_CALLBACK (popup_cmd_download_link_as) },
 	{ "BookmarkLink", STOCK_ADD_BOOKMARK, N_("_Bookmark Link..."),
 	  NULL, NULL, G_CALLBACK (popup_cmd_bookmark_link) },
@@ -328,6 +329,7 @@ struct EphyWindowPrivate
 	gboolean has_size;
 	guint num_tabs;
 	guint tab_message_cid;
+	guint help_message_cid;
 
 	guint disable_js_chrome_notifier_id;
 	guint show_toolbars_notifier_id;
@@ -859,6 +861,63 @@ init_menu_updaters (EphyWindow *window)
 			  G_CALLBACK (edit_menu_hide_cb), window);
 }
 
+static void
+menu_item_select_cb (GtkMenuItem *proxy,
+		     EphyWindow *window)
+{
+	GtkAction *action;
+	char *message;
+
+	action = g_object_get_data (G_OBJECT (proxy),  "gtk-action");
+	g_return_if_fail (action != NULL);
+	
+	g_object_get (G_OBJECT (action), "tooltip", &message, NULL);
+	if (message)
+	{
+		gtk_statusbar_push (GTK_STATUSBAR (window->priv->statusbar),
+				    window->priv->help_message_cid, message);
+		g_free (message);
+	}
+}
+
+static void
+menu_item_deselect_cb (GtkMenuItem *proxy,
+		       EphyWindow *window)
+{
+	gtk_statusbar_pop (GTK_STATUSBAR (window->priv->statusbar),
+			   window->priv->help_message_cid);
+}
+
+static void
+disconnect_proxy_cb (GtkUIManager *manager,
+		     GtkAction *action,
+		     GtkWidget *proxy,
+		     EphyWindow *window)
+{
+	if (GTK_IS_MENU_ITEM (proxy))
+	{
+		g_signal_handlers_disconnect_by_func
+			(proxy, G_CALLBACK (menu_item_select_cb), window);
+		g_signal_handlers_disconnect_by_func
+			(proxy, G_CALLBACK (menu_item_deselect_cb), window);
+	}
+}
+
+static void
+connect_proxy_cb (GtkUIManager *manager,
+		  GtkAction *action,
+		  GtkWidget *proxy,
+		  EphyWindow *window)
+{
+	if (GTK_IS_MENU_ITEM (proxy))
+	{
+		g_signal_connect (proxy, "select",
+				  G_CALLBACK (menu_item_select_cb), window);
+		g_signal_connect (proxy, "deselect",
+				  G_CALLBACK (menu_item_deselect_cb), window);
+	}
+}
+
 static void
 setup_ui_manager (EphyWindow *window)
 {
@@ -880,6 +939,11 @@ setup_ui_manager (EphyWindow *window)
 
 	merge = gtk_ui_manager_new ();
 
+	g_signal_connect (merge, "connect_proxy",
+			  G_CALLBACK (connect_proxy_cb), window);
+	g_signal_connect (merge, "disconnect_proxy",
+			  G_CALLBACK (disconnect_proxy_cb), window);
+
 	action_group = gtk_action_group_new ("WindowActions");
 	gtk_action_group_set_translation_domain (action_group, NULL);
 	gtk_action_group_add_actions (action_group, ephy_menu_entries,
@@ -1234,10 +1298,10 @@ static void
 popup_menu_at_coords (GtkMenu *menu, gint *x, gint *y, gboolean *push_in,
 		      gpointer user_data)
 {
-	EphyEmbedEvent *event = user_data;
+	EphyEmbedEvent *event = (EphyEmbedEvent *) user_data;
+
+	ephy_embed_event_get_coords (event, x, y);
 
-	*x = event->x;
-	*y = event->y;
 	*push_in = TRUE;
 }
 
@@ -1877,6 +1941,9 @@ ephy_window_init (EphyWindow *window)
 	window->priv->tab_message_cid = gtk_statusbar_get_context_id
 		(GTK_STATUSBAR (window->priv->statusbar), "tab_message");
 
+	window->priv->help_message_cid = gtk_statusbar_get_context_id
+		(GTK_STATUSBAR (window->priv->statusbar), "help_message");
+
 	/* Initialize the menus */
 	window->priv->tabs_menu = ephy_tabs_menu_new (window);
 	window->priv->fav_menu = ephy_favorites_menu_new (window);
