@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,19 +14,26 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: popup-commands.c,v 1.37 2004/01/23 22:50:02 marco Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "popup-commands.h"
 #include "ephy-shell.h"
 #include "ephy-new-bookmark.h"
+#include "ephy-embed-factory.h"
 #include "ephy-embed-persist.h"
 #include "ephy-prefs.h"
-#include "ephy-embed-utils.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-file-helpers.h"
 
 #include <string.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
+#include <gtk/gtkclipboard.h>
 
 static EphyEmbedEvent *
 get_event_info (EphyWindow *window)
@@ -41,7 +48,7 @@ get_event_info (EphyWindow *window)
 }
 
 void
-popup_cmd_link_in_new_window (EggAction *action,
+popup_cmd_link_in_new_window (GtkAction *action,
 		              EphyWindow *window)
 {
 	EphyEmbedEvent *info;
@@ -61,7 +68,7 @@ popup_cmd_link_in_new_window (EggAction *action,
 }
 
 void
-popup_cmd_link_in_new_tab (EggAction *action,
+popup_cmd_link_in_new_tab (GtkAction *action,
 		           EphyWindow *window)
 {
 	EphyEmbedEvent *info;
@@ -81,7 +88,7 @@ popup_cmd_link_in_new_tab (EggAction *action,
 }
 
 void
-popup_cmd_image_in_new_tab (EggAction *action,
+popup_cmd_image_in_new_tab (GtkAction *action,
 			    EphyWindow *window)
 {
 	EphyEmbedEvent *info;
@@ -101,7 +108,7 @@ popup_cmd_image_in_new_tab (EggAction *action,
 }
 
 void
-popup_cmd_image_in_new_window (EggAction *action,
+popup_cmd_image_in_new_window (GtkAction *action,
 			       EphyWindow *window)
 {
 	EphyEmbedEvent *info;
@@ -121,8 +128,8 @@ popup_cmd_image_in_new_window (EggAction *action,
 }
 
 void
-popup_cmd_bookmark_link (EggAction *action,
-			     EphyWindow *window)
+popup_cmd_bookmark_link (GtkAction *action,
+			 EphyWindow *window)
 {
 	GtkWidget *new_bookmark;
 	EphyBookmarks *bookmarks;
@@ -132,6 +139,7 @@ popup_cmd_bookmark_link (EggAction *action,
 	const GValue *link_rel;
 	const GValue *link;
 	const GValue *link_is_smart;
+	const GValue *linktext;
 	const char *title;
 	const char *location;
 	const char *rel;
@@ -144,15 +152,22 @@ popup_cmd_bookmark_link (EggAction *action,
 	ephy_embed_event_get_property (info, "link", &link);
 	ephy_embed_event_get_property (info, "link_title", &link_title);
 	ephy_embed_event_get_property (info, "link_rel", &link_rel);
+	ephy_embed_event_get_property (info, "linktext", &linktext);
 
-	title = g_value_get_string (link_title);
 	location = g_value_get_string (link);
+	g_return_if_fail (location);
+
 	rel = g_value_get_string (link_rel);
 	is_smart = g_value_get_int (link_is_smart);
 
-	g_return_if_fail (location);
+	title = g_value_get_string (link_title);
 
-	if (!title || !title[0])
+	if (title == NULL || title[0] == '\0')
+	{
+		title = g_value_get_string (linktext);
+	}
+
+	if (title == NULL || title[0] == '\0')
 	{
 		title = location;
 	}
@@ -171,7 +186,7 @@ popup_cmd_bookmark_link (EggAction *action,
 }
 
 void
-popup_cmd_frame_in_new_tab (EggAction *action,
+popup_cmd_frame_in_new_tab (GtkAction *action,
 			    EphyWindow *window)
 {
 	EphyTab *tab;
@@ -182,7 +197,7 @@ popup_cmd_frame_in_new_tab (EggAction *action,
 
 	embed = ephy_window_get_active_embed (window);
 
-	ephy_embed_get_location (embed, FALSE, &location);
+	location = ephy_embed_get_location (embed, FALSE);
 
 	ephy_shell_new_tab (ephy_shell, window, tab,
 			    location,
@@ -193,7 +208,7 @@ popup_cmd_frame_in_new_tab (EggAction *action,
 }
 
 void
-popup_cmd_frame_in_new_window (EggAction *action,
+popup_cmd_frame_in_new_window (GtkAction *action,
 			       EphyWindow *window)
 {
 	EphyTab *tab;
@@ -204,7 +219,7 @@ popup_cmd_frame_in_new_window (EggAction *action,
 
 	embed = ephy_window_get_active_embed (window);
 
-	ephy_embed_get_location (embed, FALSE, &location);
+	location = ephy_embed_get_location (embed, FALSE);
 
 	ephy_shell_new_tab (ephy_shell, NULL, tab,
 			    location,
@@ -224,23 +239,26 @@ popup_cmd_copy_to_clipboard (EphyWindow *window, const char *text)
 }
 
 void
-popup_cmd_copy_link_address (EggAction *action,
+popup_cmd_copy_link_address (GtkAction *action,
 			     EphyWindow *window)
 {
 	EphyEmbedEvent *event;
+	EmbedEventContext context;
 	const char *address;
 	const GValue *value;
 
 	event = get_event_info (window);
-	g_return_if_fail (IS_EPHY_EMBED_EVENT (event));
+	g_return_if_fail (EPHY_IS_EMBED_EVENT (event));
 
-	if (event->context & EMBED_CONTEXT_EMAIL_LINK)
+	context = ephy_embed_event_get_context (event);
+
+	if (context & EMBED_CONTEXT_EMAIL_LINK)
 	{
 		ephy_embed_event_get_property (event, "email", &value);
 		address = g_value_get_string (value);
 		popup_cmd_copy_to_clipboard (window, address);
 	}
-	else if (event->context & EMBED_CONTEXT_LINK)
+	else if (context & EMBED_CONTEXT_LINK)
 	{
 		ephy_embed_event_get_property (event, "link", &value);
 		address = g_value_get_string (value);
@@ -249,17 +267,15 @@ popup_cmd_copy_link_address (EggAction *action,
 }
 
 static void
-save_property_url (EggAction *action,
+save_property_url (GtkAction *action,
 		   const char *title,
 		   EphyWindow *window,
 		   gboolean ask_dest,
-		   gboolean show_progress,
 		   const char *property)
 {
 	EphyEmbedEvent *info;
 	const char *location;
 	const GValue *value;
-	GtkWidget *widget;
 	EphyEmbedPersist *persist;
 	EphyEmbed *embed;
 
@@ -270,27 +286,25 @@ save_property_url (EggAction *action,
 	ephy_embed_event_get_property (info, property, &value);
 	location = g_value_get_string (value);
 
-	widget = GTK_WIDGET (embed);
-
-	persist = ephy_embed_persist_new (embed);
+	persist = EPHY_EMBED_PERSIST
+		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
 
+	ephy_embed_persist_set_embed (persist, embed);
+	ephy_embed_persist_set_fc_title (persist, title);
+	ephy_embed_persist_set_fc_parent (persist,GTK_WINDOW (window));
+	ephy_embed_persist_set_flags
+		(persist, ask_dest ? EMBED_PERSIST_ASK_DESTINATION : 0);
+	ephy_embed_persist_set_persist_key
+		(persist, CONF_STATE_SAVE_DIR);
 	ephy_embed_persist_set_source (persist, location);
 
-	if (show_progress)
-	{
-		ephy_embed_persist_set_flags (persist,
-					      EMBED_PERSIST_SHOW_PROGRESS);
-	}
-
-	ephy_embed_utils_save (GTK_WIDGET (window), title,
-			       CONF_STATE_DOWNLOADING_DIR,
-			       ask_dest, FALSE, persist);
+	ephy_embed_persist_save (persist);
 
 	g_object_unref (G_OBJECT(persist));
 }
 
 void
-popup_cmd_open_link (EggAction *action,
+popup_cmd_open_link (GtkAction *action,
 		     EphyWindow *window)
 {
 	EphyEmbedEvent *info;
@@ -309,21 +323,26 @@ popup_cmd_open_link (EggAction *action,
 }
 
 void
-popup_cmd_download_link (EggAction *action,
+popup_cmd_download_link (GtkAction *action,
 			 EphyWindow *window)
 {
-	save_property_url (action, _("Download link"), window,
-		           eel_gconf_get_boolean
-		           (CONF_ASK_DOWNLOAD_DEST),
-		           TRUE, "link");
+	save_property_url (action, _("Download link"), window, 
+			   FALSE, "link");
 }
 
 void
-popup_cmd_save_image_as (EggAction *action,
+popup_cmd_download_link_as (GtkAction *action,
+			    EphyWindow *window)
+{
+	save_property_url (action, _("Download link"), window, 
+			   TRUE, "link");
+}
+void
+popup_cmd_save_image_as (GtkAction *action,
 			 EphyWindow *window)
 {
 	save_property_url (action, _("Save Image As"),
-			   window, TRUE, FALSE, "image");
+			   window, TRUE, "image");
 }
 
 #define CONF_DESKTOP_BG_PICTURE "/desktop/gnome/background/picture_filename"
@@ -336,23 +355,21 @@ background_download_completed (EphyEmbedPersist *persist,
 	const char *bg;
 	char *type;
 
-	ephy_embed_persist_get_dest (persist, &bg);
+	bg = ephy_embed_persist_get_dest (persist);
 	eel_gconf_set_string (CONF_DESKTOP_BG_PICTURE, bg);
 
 	type = eel_gconf_get_string (CONF_DESKTOP_BG_TYPE);
-	if (type || strcmp (type, "none") == 0)
+	if (type == NULL || strcmp (type, "none") == 0)
 	{
-		eel_gconf_set_string (CONF_DESKTOP_BG_TYPE,
-				      "wallpaper");
+		eel_gconf_set_string (CONF_DESKTOP_BG_TYPE, "wallpaper");
 	}
-
 	g_free (type);
 
 	g_object_unref (persist);
 }
 
 void
-popup_cmd_set_image_as_background (EggAction *action,
+popup_cmd_set_image_as_background (GtkAction *action,
 				   EphyWindow *window)
 {
 	EphyEmbedEvent *info;
@@ -369,27 +386,29 @@ popup_cmd_set_image_as_background (EggAction *action,
 	ephy_embed_event_get_property (info, "image", &value);
 	location = g_value_get_string (value);
 
-	persist = ephy_embed_persist_new (embed);
+	persist = EPHY_EMBED_PERSIST
+		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
 
 	base = g_path_get_basename (location);
-	dest = g_build_filename (ephy_dot_dir (),
-				 base, NULL);
+	dest = g_build_filename (ephy_dot_dir (), base, NULL);
 
-	ephy_embed_persist_set_source (persist, location);
+	ephy_embed_persist_set_embed (persist, embed);
 	ephy_embed_persist_set_dest (persist, dest);
-
-	ephy_embed_persist_save (persist);
+	ephy_embed_persist_set_flags (persist, EMBED_PERSIST_NO_VIEW);
+	ephy_embed_persist_set_source (persist, location);
 
 	g_signal_connect (persist, "completed",
 			  G_CALLBACK (background_download_completed),
 			  NULL);
 
+	ephy_embed_persist_save (persist);
+
 	g_free (dest);
 	g_free (base);
 }
 
 void
-popup_cmd_copy_image_location (EggAction *action,
+popup_cmd_copy_image_location (GtkAction *action,
 			       EphyWindow *window)
 {
 	EphyEmbedEvent *info;
@@ -407,15 +426,15 @@ popup_cmd_copy_image_location (EggAction *action,
 }
 
 void
-popup_cmd_save_background_as (EggAction *action,
+popup_cmd_save_background_as (GtkAction *action,
 			      EphyWindow *window)
 {
 	save_property_url (action, _("Save Background As"),
-			   window, TRUE, FALSE, "background_image");
+			   window, TRUE, "background_image");
 }
 
 void
-popup_cmd_open_frame (EggAction *action,
+popup_cmd_open_frame (GtkAction *action,
 		      EphyWindow *window)
 {
 	char *location;
@@ -424,7 +443,7 @@ popup_cmd_open_frame (EggAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_embed_get_location (embed, FALSE, &location);
+	location = ephy_embed_get_location (embed, FALSE);
 
 	ephy_embed_load_url (embed, location);
 
@@ -432,7 +451,7 @@ popup_cmd_open_frame (EggAction *action,
 }
 
 void
-popup_cmd_open_image (EggAction *action,
+popup_cmd_open_image (GtkAction *action,
 		      EphyWindow *window)
 {
 	EphyEmbedEvent *info;
